@@ -1,6 +1,6 @@
 /*
  * rd_stats.c: Read system statistics
- * (C) 1999-2009 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 1999-2011 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -41,7 +41,6 @@
 #define _(string) (string)
 #endif
 
-
 /*
  ***************************************************************************
  * Read CPU statistics and machine uptime.
@@ -161,8 +160,8 @@ void read_stat_cpu(struct stats_cpu *st_cpu, int nbr,
 
 /*
  ***************************************************************************
- * Read processes (tasks) creation and context switches statistics from
- * /proc/stat
+ * Read processes (tasks) creation and context switches statistics
+ * from /proc/stat.
  *
  * IN:
  * @st_pcsw	Structure where stats will be saved.
@@ -238,7 +237,7 @@ void read_stat_irq(struct stats_irq *st_irq, int nbr)
 
 /*
  ***************************************************************************
- * Read queue and load statistics from /proc/loadavg.
+ * Read queue and load statistics from /proc/loadavg and /proc/stat.
  *
  * IN:
  * @st_queue	Structure where stats will be saved.
@@ -250,6 +249,7 @@ void read_stat_irq(struct stats_irq *st_irq, int nbr)
 void read_loadavg(struct stats_queue *st_queue)
 {
 	FILE *fp;
+	char line[8192];
 	int load_tmp[3];
 
 	if ((fp = fopen(LOADAVG, "r")) == NULL)
@@ -273,6 +273,21 @@ void read_loadavg(struct stats_queue *st_queue)
 		/* Do not take current process into account */
 		st_queue->nr_running--;
 	}
+
+	/* Read nr of tasks blocked from /proc/stat */
+	if ((fp = fopen(STAT, "r")) == NULL)
+		return;
+
+	while (fgets(line, 8192, fp) != NULL) {
+
+		if (!strncmp(line, "procs_blocked ", 14)) {
+			/* Read number of processes blocked */
+			sscanf(line + 14, "%lu", &st_queue->procs_blocked);
+			break;
+		}
+	}
+
+	fclose(fp);
 }
 
 /*
@@ -316,6 +331,14 @@ void read_meminfo(struct stats_memory *st_memory)
 			/* Read the amount of cached swap in kB */
 			sscanf(line + 11, "%lu", &st_memory->caskb);
 		}
+		else if (!strncmp(line, "Active:", 7)) {
+			/* Read the amount of active memory in kB */
+			sscanf(line + 7, "%lu", &st_memory->activekb);
+		}
+		else if (!strncmp(line, "Inactive:", 9)) {
+			/* Read the amount of inactive memory in kB */
+			sscanf(line + 9, "%lu", &st_memory->inactkb);
+		}
 		else if (!strncmp(line, "SwapTotal:", 10)) {
 			/* Read the total amount of swap memory in kB */
 			sscanf(line + 10, "%lu", &st_memory->tlskb);
@@ -516,7 +539,7 @@ void read_diskstats_disk(struct stats_disk *st_disk, int nbr, int read_part)
 			   &tot_ticks, &rq_ticks) == 11) {
 			
 			if (!rd_ios && !wr_ios)
-				/* Unused device: ignore it */
+				/* Unused device: Ignore it */
 				continue;
 			if (read_part || is_device(dev_name)) {
 				st_disk_i = st_disk + dsk++;
@@ -550,8 +573,6 @@ void read_diskstats_disk(struct stats_disk *st_disk, int nbr, int read_part)
  */
 void read_tty_driver_serial(struct stats_serial *st_serial, int nbr)
 {
-#ifndef SMP_RACE
-	
 	FILE *fp;
 	struct stats_serial *st_serial_i;
 	int sl = 0;
@@ -597,7 +618,6 @@ void read_tty_driver_serial(struct stats_serial *st_serial, int nbr)
 	}
 
 	fclose(fp);
-#endif
 }
 
 /*
@@ -1571,6 +1591,157 @@ void read_net_udp6(struct stats_net_udp6 *st_net_udp6)
 	fclose(fp);
 }
 
+/*
+ ***************************************************************************
+ * Read CPU frequency statistics.
+ *
+ * IN:
+ * @st_pwr_cpufreq	Structure where stats will be saved.
+ * @nbr			Total number of CPU (including cpu "all").
+ *
+ * OUT:
+ * @st_pwr_cpufreq	Structure with statistics.
+ ***************************************************************************
+ */
+void read_cpuinfo(struct stats_pwr_cpufreq *st_pwr_cpufreq, int nbr)
+{
+	FILE *fp;
+	struct stats_pwr_cpufreq *st_pwr_cpufreq_i;
+	char line[1024];
+	int proc_nb = 0, nr = 0;
+	unsigned int ifreq, dfreq;
+	
+	if ((fp = fopen(CPUINFO, "r")) == NULL)
+		return;
+	
+	st_pwr_cpufreq->cpufreq = 0;
+	
+	while (fgets(line, 1024, fp) != NULL) {
+		
+		if (!strncmp(line, "processor\t", 10)) {
+			sscanf(strchr(line, ':') + 1, "%d", &proc_nb);
+		}
+		
+		else if (!strncmp(line, "cpu MHz\t", 8)) {
+			sscanf(strchr(line, ':') + 1, "%u.%u", &ifreq, &dfreq);
+			
+			if (proc_nb < (nbr - 1)) {
+				/* Save current CPU frequency */
+				st_pwr_cpufreq_i = st_pwr_cpufreq + proc_nb + 1;
+				st_pwr_cpufreq_i->cpufreq = ifreq * 100 + dfreq / 10;
+				
+				/* Also save it to compute an average CPU frequency */
+				st_pwr_cpufreq->cpufreq += st_pwr_cpufreq_i->cpufreq;
+				nr++;
+			}
+			else if (!proc_nb && (nbr == 1)) {
+				/*
+				 * We are reading freq for "Processor 0" and we have a machine
+				 * with only one processor and not an SMP kernel, with /sys not mounted
+				 * (the nr of proc has been counted using /proc/stat and there was
+				 * only one line with global CPU stats here).
+				 * This is a very specific case, I must admit...
+				 */
+				st_pwr_cpufreq->cpufreq = ifreq * 100 + dfreq / 10;
+			}
+		}
+	}
+	
+	fclose(fp);
+	
+	if (nr) {
+		/* Compute average CPU frequency for this machine */
+		st_pwr_cpufreq->cpufreq /= nr;
+	}
+}
+
+/*
+ ***************************************************************************
+ * Read hugepages statistics from /proc/meminfo.
+ *
+ * IN:
+ * @st_huge	Structure where stats will be saved.
+ *
+ * OUT:
+ * @st_huge	Structure with statistics.
+ ***************************************************************************
+ */
+void read_meminfo_huge(struct stats_huge *st_huge)
+{
+	FILE *fp;
+	char line[128];
+	unsigned long szhkb = 0;
+
+	if ((fp = fopen(MEMINFO, "r")) == NULL)
+		return;
+
+	while (fgets(line, 128, fp) != NULL) {
+
+		if (!strncmp(line, "HugePages_Total:", 16)) {
+			/* Read the total number of huge pages */
+			sscanf(line + 16, "%lu", &st_huge->tlhkb);
+		}
+		else if (!strncmp(line, "HugePages_Free:", 15)) {
+			/* Read the number of free huge pages */
+			sscanf(line + 15, "%lu", &st_huge->frhkb);
+		}
+		else if (!strncmp(line, "Hugepagesize:", 13)) {
+			/* Read the default size of a huge page in kB */
+			sscanf(line + 13, "%lu", &szhkb);
+		}
+	}
+
+	fclose(fp);
+
+	/* We want huge pages stats in kB and not expressed in a number of pages */
+	st_huge->tlhkb *= szhkb;
+	st_huge->frhkb *= szhkb;
+}
+
+/*
+ ***************************************************************************
+ * Read CPU average frequencies statistics.
+ *
+ * IN:
+ * @st_pwr_wghfreq	Structure where stats will be saved.
+ * @cpu_nr		CPU number for which time_in_state date will be read.
+ * @nbr			Total number of states (frequencies).
+ *
+ * OUT:
+ * @st_pwr_wghfreq	Structure with statistics.
+ ***************************************************************************
+ */
+void read_time_in_state(struct stats_pwr_wghfreq *st_pwr_wghfreq, int cpu_nr, int nbr)
+{
+	FILE *fp;
+	struct stats_pwr_wghfreq *st_pwr_wghfreq_j;
+	char filename[MAX_PF_NAME];
+	char line[128];
+	int j = 0;
+	unsigned long freq;
+	unsigned long long time_in_state;
+
+	snprintf(filename, MAX_PF_NAME, "%s/cpu%d/%s",
+		 SYSFS_DEVCPU, cpu_nr, SYSFS_TIME_IN_STATE);
+	if ((fp = fopen(filename, "r")) == NULL)
+		return;
+
+	while (fgets(line, 128, fp) != NULL) {
+
+		sscanf(line, "%lu %llu", &freq, &time_in_state);
+
+		if (j < nbr) {
+			/* Save current frequency and time */
+			st_pwr_wghfreq_j = st_pwr_wghfreq + j;
+			st_pwr_wghfreq_j->freq = freq;
+			st_pwr_wghfreq_j->time_in_state = time_in_state;
+			j++;
+		}
+	}
+
+	fclose(fp);
+}
+
 /*
  ***************************************************************************
  * Read machine uptime, independently of the number of processors.
@@ -1592,7 +1763,8 @@ void read_uptime(unsigned long long *uptime)
 		return;
 
 	sscanf(line, "%lu.%lu", &up_sec, &up_cent);
-	*uptime = (unsigned long long) up_sec * HZ + (unsigned long long) up_cent * HZ / 100;
+	*uptime = (unsigned long long) up_sec * HZ +
+	          (unsigned long long) up_cent * HZ / 100;
 
 	fclose(fp);
 
@@ -1615,11 +1787,11 @@ int get_irq_nr(void)
 
 	if ((fp = fopen(STAT, "r")) == NULL)
 		return 0;
-	
+
 	while (fgets(line, 8192, fp) != NULL) {
 
 		if (!strncmp(line, "intr ", 5)) {
-			
+
 			while (pos < strlen(line)) {
 				in++;
 				pos += strcspn(line + pos + 1, " ") + 1;
@@ -1628,7 +1800,7 @@ int get_irq_nr(void)
 	}
 
 	fclose(fp);
-	
+
 	return in;
 }
 
@@ -1642,16 +1814,6 @@ int get_irq_nr(void)
  ***************************************************************************
  */
 int get_serial_nr(void)
-#ifdef SMP_RACE
-{
-	/*
-	 * Ignore serial lines if SMP_RACE flag is defined.
-	 * This is because there is an SMP race in some 2.2.x kernels that
-	 * may be triggered when reading the /proc/tty/driver/serial file.
-	 */
-	return 0;
-}
-#else
 {
 	FILE *fp;
 	char line[256];
@@ -1674,7 +1836,6 @@ int get_serial_nr(void)
 
 	return sl;
 }
-#endif
 
 /*
  ***************************************************************************
@@ -1769,7 +1930,7 @@ int get_diskstats_dev_nr(int count_part, int only_used_dev)
 int get_disk_nr(unsigned int f)
 {
 	int disk_nr;
-	
+
 	/*
 	 * Partitions are taken into account by sar -d only with
 	 * kernels 2.6.25 and later.
@@ -1786,7 +1947,7 @@ int get_disk_nr(unsigned int f)
  * RETURNS:
  * Number of processors (online and offline).
  * A value of 0 means that /sys was not mounted.
- * A value of N (!=0) means N processor(s) (0 .. N-1).
+ * A value of N (!=0) means N processor(s) (cpu0 .. cpu(N-1)).
  ***************************************************************************
  */
 int get_sys_cpu_nr(void)
@@ -1928,7 +2089,7 @@ int get_irqcpu_nr(char *file, int max_nr_irqcpu, int cpu_nr)
 	}
 
 	fclose(fp);
-	
+
 	if (line) {
 		free(line);
 	}
@@ -1938,54 +2099,29 @@ int get_irqcpu_nr(char *file, int max_nr_irqcpu, int cpu_nr)
 
 /*
  ***************************************************************************
- * Read CPU frequency statistics.
+ * Count number of possible frequencies for CPU#0.
  *
- * IN:
- * @st_pwr_cpufreq	Structure where stats will be saved.
- * @nbr			Total number of CPU (including cpu "all").
- *
- * OUT:
- * @st_pwr_cpufreq	Structure with statistics.
+ * RETURNS:
+ * Number of frequencies.
  ***************************************************************************
  */
-void read_cpuinfo(struct stats_pwr_cpufreq *st_pwr_cpufreq, int nbr)
+int get_freq_nr(void)
 {
 	FILE *fp;
-	struct stats_pwr_cpufreq *st_pwr_cpufreq_i;
-	char line[1024];
-	int proc_nb = 0, nr = 0;
-	unsigned int ifreq, dfreq;
-	
-	if ((fp = fopen(CPUINFO, "r")) == NULL)
-		return;
-	
-	st_pwr_cpufreq->cpufreq = 0;
-	
-	while (fgets(line, 1024, fp) != NULL) {
-		
-		if (!strncmp(line, "processor\t", 10)) {
-			sscanf(strchr(line, ':') + 1, "%d", &proc_nb);
-		}
-		
-		else if (!strncmp(line, "cpu MHz\t", 8)) {
-			sscanf(strchr(line, ':') + 1, "%u.%u", &ifreq, &dfreq);
-			
-			if (proc_nb < (nbr - 1)) {
-				/* Save current CPU frequency */
-				st_pwr_cpufreq_i = st_pwr_cpufreq + proc_nb + 1;
-				st_pwr_cpufreq_i->cpufreq = ifreq * 100 + dfreq / 10;
-				
-				/* Also save it to compute an average CPU frequency */
-				st_pwr_cpufreq->cpufreq += st_pwr_cpufreq_i->cpufreq;
-				nr++;
-			}
-		}
+	char filename[MAX_PF_NAME];
+	char line[128];
+	int freq = 0;
+
+	snprintf(filename, MAX_PF_NAME, "%s/cpu0/%s",
+		 SYSFS_DEVCPU, SYSFS_TIME_IN_STATE);
+	if ((fp = fopen(filename, "r")) == NULL)
+		return 0;	/* No time_in_state file for CPU#0 */
+
+	while (fgets(line, 128, fp) != NULL) {
+		freq++;
 	}
-	
+
 	fclose(fp);
-	
-	if (nr) {
-		/* Compute average CPU frequency for this machine */
-		st_pwr_cpufreq->cpufreq /= nr;
-	}
+
+	return freq;
 }
